package cz.zweistein.df.soundsense.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

import cz.zweistein.df.soundsense.util.LoggerSource;

public class SoundsXML {
	private static Logger logger = LoggerSource.logger;
	
	private List<Sound> sounds;

	public SoundsXML(String directory) {
		
		this.sounds = new LinkedList<Sound>();
		this.loadDirectory(directory);
		
	}
	
	private void loadDirectory(String directory) {
		logger.info("loading directory "+directory);
		File dir = new File(directory);
		
		String[] files = dir.list();
		Arrays.sort(files);
		
		if (files == null) {
			logger.info(directory+" is empty or invalid?");
		} else {
			for (int i = 0; i < files.length; i++) {
				
				String fileName = files[i];
				
				if (new File(directory+fileName).isDirectory()) {
					this.loadDirectory(directory+fileName+"/");
				} else if (fileName.endsWith(".xml")) {
				
					try {
						logger.info("Loading config "+directory+fileName);
						this.loadFile(directory+fileName);
					} catch (Exception e) {
						logger.severe("Failed to load "+ fileName + ": " + e.toString());
					}
				
				}
			}
		}
		
	}
	
	private void loadFile(String fileName) throws SAXException, IOException {
		
		Document doc = null;
		InputSource source = new InputSource(new FileInputStream(fileName));
		DOMParser parser = new DOMParser();
		parser.parse(source);
		doc = parser.getDocument();
		
		NodeList soundTags = doc.getElementsByTagName("sound");
		
		for (int i = 0; i < soundTags.getLength(); i++) {
			Node soundNode = soundTags.item(i);
			
			String logPattern = soundNode.getAttributes().getNamedItem("logPattern").getNodeValue();
			
			boolean isMusic = false;
			if (soundNode.getAttributes().getNamedItem("music") != null) {
				String music = soundNode.getAttributes().getNamedItem("music").getNodeValue();
				if ("true".equals(music)) {
					isMusic = true;
				}
			}
			
			boolean isLoop = false;
			if (soundNode.getAttributes().getNamedItem("loop") != null) {
				String loop = soundNode.getAttributes().getNamedItem("loop").getNodeValue();
				if ("true".equals(loop)) {
					isLoop = true;
				}
			}
			
			List<String> soundFiles = new ArrayList<String>();
			
			NodeList soundElements = soundNode.getChildNodes();
			for (int j = 0; j < soundElements.getLength(); j++) {
				Node configNode = soundElements.item(j);
				String name = configNode.getLocalName();
				
				if ("soundFile".equals(name)) {
					String soundFile = new File(fileName).getParent()+"/"+configNode.getAttributes().getNamedItem("fileName").getNodeValue();
					if (new File(soundFile).exists()) {
						soundFiles.add(soundFile);
					} else {
						logger.info("Did not find "+soundFile+", ignoring.");
					}
				}
			}
			
			if (soundFiles.size() == 0) {
				logger.info("Sound for '"+logPattern+"' does not contain any soundFiles, ignoring.");
			} else {
				this.sounds.add(new Sound(soundFiles, logPattern, isLoop, isMusic));
			}
		}
		
	}

	public List<Sound> getSounds() {
		return this.sounds;
	}
	
}
